defmodule NullaWeb.InboxController do
  use NullaWeb, :controller
  alias Nulla.HTTPSignature
  alias Nulla.ActivityPub
  alias Nulla.Utils
  alias Nulla.Models.Actor
  alias Nulla.Models.Relation

  def inbox(
        conn,
        %{"id" => follow_id, "type" => "Follow", "actor" => actor_uri, "object" => target_uri}
      ) do
    with {:ok, target_actor} <- Utils.resolve_local_actor(target_uri),
         {:ok, remote_actor_json} <- Utils.fetch_remote_actor(actor_uri),
         :ok <- HTTPSignature.verify(conn, remote_actor_json),
         remote_actor <-
           Actor.create_actor(
             remote_actor_json
             |> Map.put("ap_id", remote_actor_json["id"])
             |> Map.delete("id")
             |> Map.put("domain", URI.parse(remote_actor_json["id"]).host)
           ),
         follow_activity <-
           Activity.create_activity(%{
             ap_id: follow_id,
             type: "Follow",
             actor: actor_uri,
             object: target_uri
           }),
         accept_activity <-
           Activity.create_activity(%{
             type: "Accept",
             actor: target_uri,
             object: follow_activity
           }),
         relation <- Relation.create_relation(%{
           id: 1,
           follower: remote_actor.id,
           followed: target_actor.id
         }) do
      conn
      |> put_resp_content_type("application/activity+json")
      |> send_resp(
        200,
        Jason.encode!(ActivityPub.follow_accept(accept_activity))
      )
    else
      error ->
        IO.inspect(error, label: "Follow error")
        json(conn, %{"error" => "Failed to process Follow"})
    end
  end
end
